#include "opencv2/opencv.hpp"
#include "opencv2/videoio.hpp"
#include "opencv2/imgproc.hpp"
#include "opencv2/highgui.hpp"
#include <vector>
#include <math.h>
# define LengthFishPixel    75.0 // Define length of the larva in pixel:

using namespace std;
using namespace cv;
double ComputeTailDeflection(Mat img);

// Define Structure useful for ellipse manipulation:
struct ImageMoment
{
	double m00;
	double m10;
	double m01;
	double mu20;
	double mu02;
	double mu11;
};
struct EllipseShape
{
	double CenterX;
	double CenterY;
	double Theta;
	double L;
	double l;
};
ImageMoment ComputeMomentPoint(cv::Mat Coord);
ImageMoment ComputeMoment(vector<int> &X,vector<int> &Y);
EllipseShape ComputeEllipseParameter(ImageMoment M);


//=================== Image Processing Subroutine ===========================================

double ComputeTailDeflection(Mat img)
{
    cv::Mat mtx(img);
    cv::Mat Point0,Point1,Point2;
	findNonZero(mtx, Point0);

	ImageMoment M0;
	EllipseShape E0;

	M0=ComputeMomentPoint(Point0);//(X0,Y0);

	E0=ComputeEllipseParameter(M0);
	// Orient Tail left, Head Right:
	if (cos(E0.Theta)<0) {cout<<"On tourne"<<endl;E0.Theta=E0.Theta+M_PI;}

	// SPACE PARTITIONNING:
	vector<int> X1;
	X1.reserve(floor(Point0.total()/2.0));
	vector<int> Y1;
	Y1.reserve(floor(Point0.total()/2.0));
	vector<int> X2;
	X2.reserve(floor(Point0.total()/2.0));
	vector<int> Y2;
	Y2.reserve(floor(Point0.total()/2.0));
    int i;
	for(i=0;i<Point0.total();i++){

		if (((double)Point0.at<cv::Point>(i).x-E0.CenterX)*cos(E0.Theta)+((double)Point0.at<cv::Point>(i).y-E0.CenterY)*sin(E0.Theta)>0){
			X1.push_back(Point0.at<cv::Point>(i).x);
			Y1.push_back(Point0.at<cv::Point>(i).y);
		}
		else{
			X2.push_back(Point0.at<cv::Point>(i).x);
			Y2.push_back(Point0.at<cv::Point>(i).y);
		}
	}

	// Head
	ImageMoment M1;
	M1=ComputeMoment(X1,Y1);
	EllipseShape E1;
	E1=ComputeEllipseParameter(M1);

	// Tail
	ImageMoment M2;
	M2=ComputeMoment(X2,Y2);
	EllipseShape E2;
	E2=ComputeEllipseParameter(M2);

	// CURVATURE
	double det=sin(E1.Theta)*cos(E2.Theta)-cos(E1.Theta)*sin(E2.Theta);
	double t_1=1/det*(-cos(E2.Theta)*(E2.CenterX-E1.CenterX)-sin(E2.Theta)*(E2.CenterY-E1.CenterY));

	double CenterCurvX=E1.CenterX+t_1*-sin(E1.Theta);
	double CenterCurvY=E1.CenterY+t_1*cos(E1.Theta);

	double Rmoy=0;

	for(i=0;i<Point0.total();i++){

		Rmoy+=pow( pow(Point0.at<cv::Point>(i).x-CenterCurvX,2)+pow(Point0.at<cv::Point>(i).y-CenterCurvY,2),0.5);
	}
	Rmoy=Rmoy/Point0.total();

	bool sign=(E1.CenterX-CenterCurvX)*(E2.CenterY-CenterCurvY)-(E2.CenterX-CenterCurvX)*(E1.CenterY-CenterCurvY)>0;

	double Curv=-1/Rmoy*(2*sign-1)*LengthFishPixel;

	X1.clear();
	Y1.clear();
	X2.clear();
	Y2.clear();

    return Curv;
}

ImageMoment ComputeMomentPoint(cv::Mat Coord)//vector<int> &X,vector<int> &Y)
{
	int l,k;
	ImageMoment M;
	M.m00=Coord.total();//X.size();
	M.m10=0;
	M.m01=0;
	M.mu11=0;
	M.mu20=0;
	M.mu02=0;
	//cout<<X.size()<<endl;
	for(l=0;l<Coord.total();l++)
	{
		M.m10+=Coord.at<cv::Point>(l).x;
		//pow((double)X.at(l),1)*pow((double)Y.at(l),0);
		M.m01+=Coord.at<cv::Point>(l).y;//pow((double)X.at(l),0)*pow((double)Y.at(l),1);
	}
	for(l=0;l<Coord.total();l++)
	{
		M.mu11+=(Coord.at<cv::Point>(l).x-M.m10/M.m00)*(Coord.at<cv::Point>(l).y-M.m01/M.m00);//pow((double)X.at(l)-M.m10/M.m00,1)*pow((double)Y.at(l)-M.m01/M.m00,1);
		M.mu20+=pow(Coord.at<cv::Point>(l).x-M.m10/M.m00,2);//pow((double)X.at(l)-M.m10/M.m00,2)*pow((double)Y.at(l)-M.m01/M.m00,0);
		M.mu02+=pow(Coord.at<cv::Point>(l).y-M.m01/M.m00,2);//pow((double)X.at(l)-M.m10/M.m00,0)*pow((double)Y.at(l)-M.m01/M.m00,2);
	}
	return M;
}

ImageMoment ComputeMoment(vector<int> &X,vector<int> &Y)
{
	int l,k;
	ImageMoment M;
	M.m00=X.size();
	M.m10=0;
	M.m01=0;
	M.mu11=0;
	M.mu20=0;
	M.mu02=0;
	//cout<<X.size()<<endl;
	for(l=0;l<X.size();l++)
	{
		M.m10+=(double)X.at(l);
		M.m01+=(double)Y.at(l);
	}
	for(l=0;l<X.size();l++)
	{
		M.mu11+=pow((double)X.at(l)-M.m10/M.m00,1)*pow((double)Y.at(l)-M.m01/M.m00,1);
		M.mu20+=pow((double)X.at(l)-M.m10/M.m00,2);
		M.mu02+=pow((double)Y.at(l)-M.m01/M.m00,2);
	}
	return M;
}

EllipseShape ComputeEllipseParameter(ImageMoment M)
{
	EllipseShape E;
	double I1,I2;
	I1=((M.mu20+M.mu02)+sqrt(pow(M.mu20-M.mu02,2)+4*pow(M.mu11,2)))/2;
	I2=((M.mu20+M.mu02)-sqrt(pow(M.mu20-M.mu02,2)+4*pow(M.mu11,2)))/2;

	E.CenterX=M.m10/M.m00;
	E.CenterY=M.m01/M.m00;
	E.Theta=0.5*atan2(2*M.mu11,M.mu20-M.mu02);
	//E.Dir=[cos(E.Theta+3.1416/2),sin(E.Theta+3.1416/2)];
	E.l=2*sqrt(I2/M.m00);
	E.L=2*sqrt(I1/M.m00);
	return E;

}

